(function ($) {
  'use strict';

  Drupal.behaviors.euccx = {
    attach: function (context, settings) {

      $(window).load(function() {
        Drupal.euccx.init();
      });

    }
  };

  Drupal.euccx = {};

  // Entry function for our script.
  // Delegates the jobs to other functions.
  Drupal.euccx.init = function() {
    Drupal.euccx.handleBanner();
    Drupal.euccx.handleCookieSettingsPage();
    Drupal.euccx.processStatus(Drupal.euccx.getCookieStatus());
  }

  // When the user clicks on the agree button of the EU Cookie Compliance module
  // we consider it a "carte blanche" consent for all cookies that are defined
  // by our plugins. Instead of waiting for the next page refresh, we can attach
  // an event that unblocks everything right here, right now.
  Drupal.euccx.handleBanner = function() {
    $(document).ready(function() {
      $('.agree-button').click(function() {
        Drupal.euccx.unblockAll();
        Drupal.euccx.yettUnblockAll();
      });
    });
  }

  // If we are in the page where the cookie settings' block is included, we need
  // to do some stuff.
  Drupal.euccx.handleCookieSettingsPage = function() {
    var cookieBlock = $('#cookie-tabs');
    // If the cookie management block is not in the page, exit.
    // No need to run extra code for something that is not even in the page
    if (!cookieBlock.length)
      return;

    // Create the jquery ui tabs for the cookie management block
    $('#cookie-tabs').tabs();
    $('#sliding-popup').hide();
    // ... handle the switches
    Drupal.euccx.handleSwitches();
    // ... and attach functionality for the save key
    Drupal.euccx.handleCookieSettingsBehaviors();
  }

  // The function that decides what to do with the current user's status.
  // Responsible for unblocking javascript when needed.
  Drupal.euccx.processStatus = function(status) {
    var offendingCookies = [];
    // If the main eucc cookie is not set, it means that the user has not agreed
    // to anything yet. Move on.
    if (!status['cookie-eucc']) {
      Drupal.euccx.handleBlockOverrides('');
      return;
    }

    // If the euccx cookie was never set but the eucc cookie was set (checked
    // for that above) it means that the user agreed to everything so we need to
    // unblock everything and move on.
    if (!status['cookie-euccx']) {
      Drupal.euccx.unblockAll();
      Drupal.euccx.yettUnblockAll();
      return;
    }

    // We are now in the case where both cookies are set. We need to surgically
    // unblock code depending on its plugin's settings.
    var pluginNames = Drupal.euccx.getPluginNames();
    var settings = Drupal.settings.euccx;
    for (var i = 0; i < pluginNames.length; i++) {
      var currentPlugin = pluginNames[i];
      // If current plugin is not in the list of the plugins that the user chose
      // to keep enabled, we move on to the next plugin.
      if (status['cookie-euccx'].indexOf(currentPlugin) === -1) {
        Drupal.euccx.handleBlockOverrides(currentPlugin);
        continue;
      }


      // If the specific plugin has a YETT blacklist defined, we need to get it
      // and whitelist the set of rules
      var blacklist = false;
      if (settings['plugins'] && settings['plugins'][currentPlugin] &&
        settings['plugins'][currentPlugin]['blacklist'])
        blacklist = settings['plugins'][currentPlugin]['blacklist'];

      if (blacklist) {
        Drupal.euccx.yettUnblockPlugin(blacklist);
      }

      // If the specific plugin has a js_exclude list, there will be a function 
      // defined that we need to call in order to run the scripts.
      var js_exclude = false;
      if (settings['plugins'] && settings['plugins'][currentPlugin] &&
        settings['plugins'][currentPlugin]['js_exclude'])
        js_exclude = settings['plugins'][currentPlugin]['js_exclude'];

      if (js_exclude) {
        Drupal.euccx.unblockPlugin(currentPlugin);
      }

    }
  }

  // Check whether our plugin has defined any block overrides. If it did, watch
  // the DOM for the selector and act accordingly.
  Drupal.euccx.handleBlockOverrides = function(plugin) {
    $(document).ready(function() {
      var settings = Drupal.settings.euccx;
      if (settings && settings['plugins']) {
        var plugins = [];
        if (plugin === '') {
          for (var pluginName in settings['plugins']) {
            if (!settings.plugins.hasOwnProperty(pluginName))
              continue;
            plugins.push(pluginName);
          }
        } else {
          if (settings.plugins.hasOwnProperty(plugin))
            plugins.push(plugin);
        }
        for (var i = 0; i < plugins.length; i++) {
          if (!settings['plugins'][plugins[i]].hasOwnProperty('overrides'))
            continue;
          for (var key in settings['plugins'][plugins[i]]['overrides']) {
            if (!settings['plugins'][plugins[i]]['overrides']
              .hasOwnProperty(key))
              continue;
            $(key)
              .empty()
              .append(settings['plugins'][plugins[i]]['overrides'][key]);
          }
        }
      }
    });
  }

  // Make sure that when the user hits the save button, it actually does
  // something.
  Drupal.euccx.handleCookieSettingsBehaviors = function() {
    $('body').on('click', '.cookie-settings-save', function(e) {
      try {
        e.preventDefault();
        // Make sure that the EU Cookie Compliance Module has set its cookie
        Drupal.eu_cookie_compliance.acceptAction();
        var plugin;
        // Create our cookie content
        // We need to at least have some content in the cookie in order to make
        // sure that we can check its contents easier later (if for example the
        // cookie was empty, it would not have been possible to check the user's
        // settings with a simple !status['cookie-euccx'].
        var cookieData = 'cookies-enabled:';
        $('#cookie-tabs input').each(function() {
          if ($(this).prop('checked')) {
            plugin = $(this).attr("name").match(/-[^-]+$/)[0].substring(1);
            cookieData += plugin + "|";
          }
        });
        // ... set our cookie
        Drupal.euccx.setCookie(cookieData);
        // ... and reload the page so that the user gets some feedback of
        // something happening and if for example he enabled google analytics it
        // records the visit.
        location.reload();
      } catch (e) {
        // console.log(e);
      }
    });
  }

  // Check the cookies that are stored in the user's browser and a return an
  // object with all the relevant information.
  Drupal.euccx.getCookieStatus = function() {
    var status = {};
    var cookieName = (typeof eu_cookie_compliance_cookie_name === 'undefined' ||
      eu_cookie_compliance_cookie_name === '') ? 
      'cookie-agreed' : 
      eu_cookie_compliance_cookie_name;
    var cookieEUCC = Drupal.euccx.getCookie(cookieName);
    var cookieEUCCX = Drupal.euccx.getCookie('cookie-extras-agreed');

    status['cookie-eucc'] = cookieEUCC;
    status['cookie-euccx'] = cookieEUCCX;
    if (typeof cookieEUCCX === 'string') {
      var pluginNames = Drupal.euccx.getPluginNames();
      for (var i = 0; i < pluginNames.length; i++) {
        status[pluginNames[i]] = (cookieEUCCX.indexOf(pluginNames[i]) !== -1);
      }
    }
    return status;
  }

  // Unblocks all plugins that used yett
  Drupal.euccx.yettUnblockAll = function() {
    if (!window.yett)
      return;
    window.yett.unblock();
  }

  // Whitelist a specific set of scripts that were blocked via yett
  Drupal.euccx.yettUnblockPlugin = function(rules) {
    if (!window.yett)
      return;
    // This should work now after a modification was made to the yett library.
    // See: https://github.com/snipsco/yett/issues/11 for more info on what the
    // problem was and how it was solved in 0.1.8 version of yett.
    window.yett.unblock(rules);
  }

  // Unblock all plugins
  Drupal.euccx.unblockAll = function() {
    var plugins = Drupal.euccx.getPluginNames();
    for (var i = 0; i < plugins.length; i++) {
      Drupal.euccx.unblockPlugin(plugins[i]);
    }
  }

  // Given a plugin name execute the function that will unblock its code
  Drupal.euccx.unblockPlugin = function(pluginName) {
    var functionName = 'euccx' + pluginName + 'Load';
    try {
      window[functionName]();
    } catch(e) {
      // console.log(e); 
    }
  }

  // Return an array of the plugin names that are defined
  Drupal.euccx.getPluginNames = function() {
    if (Drupal.settings.euccx && Drupal.settings.euccx.plugins) {
      var pluginNames = [];
      for (var key in Drupal.settings.euccx.plugins) {
        if (Drupal.settings.euccx.plugins.hasOwnProperty(key)) {
          pluginNames.push(key);
        }
      }
      return pluginNames;
    }
    return false;
  }

  // Return false when the cookie is not properly set or the cookie value when
  // it is (properly set).
  Drupal.euccx.getCookie = function(cookieName) {
    var cookieValue = $.cookie(cookieName);
    if (typeof(cookieValue) === 'undefined' || (cookieValue === null))
      return false;
    return cookieValue;
  }

  // This is using the exact same code with EU Cookie since our cookie is also
  // respecting most of the settings that the user supplied in EU Cookie
  // Compliance module.
  Drupal.euccx.setCookie = function(content) {
    try {
      var date = new Date();
      var domain = Drupal.settings.eu_cookie_compliance.domain ? 
        Drupal.settings.eu_cookie_compliance.domain : 
        '';
      var path = Drupal.settings.basePath;
      var cookieName = 'cookie-extras-agreed';
      if (path.length > 1) {
        var pathEnd = path.length - 1;
        if (path.lastIndexOf('/') === pathEnd) {
          path = path.substring(0, pathEnd);
        }
      }
      var cookieSession = parseInt(
        Drupal.settings.eu_cookie_compliance.cookie_session
      );

      if (cookieSession) {
        $.cookie(cookieName, content, { path: path, domain: domain });
      } else {
        var lifetime = parseInt(
          Drupal.settings.eu_cookie_compliance.cookie_lifetime
        );
        date.setDate(date.getDate() + lifetime);
        $.cookie(cookieName, content, { 
          expires: date, 
          path: path, 
          domain: domain 
        });
      }

      // Store consent if applicable.
      if (Drupal.settings.eu_cookie_compliance.store_consent) {
        var url = Drupal.settings.basePath;
        url += Drupal.settings.pathPrefix;
        url += 'eu-cookie-compliance/store_consent/banner';

        $.post(url, {}, function (data) { });
      }
    } catch(e) {
      // console.log(e);
    }
  }

  // The switches in the block are always set to off. It's up to us to turn them
  // on, if the cookie stored tells us so.
  Drupal.euccx.handleSwitches = function() {
    // If the cookie is not set, just turn them on by default
    var cookieValue = Drupal.euccx.getCookie('cookie-extras-agreed');
    if (!cookieValue) {
      $('#cookie-tabs input').prop('checked', true);
    } else {
      var plugins = Drupal.settings.euccx.plugins;
      for (var key in plugins) {
        if (plugins.hasOwnProperty(key)) {
          var input = $("input[name*='" + key + "']");
          if (input.length && (cookieValue.indexOf(key) !== -1)) {
            input.prop('checked', true);
          }
        }
      }
    }
  }

  // Check our settings for cookies that plugins declared as being offensive.
  // If the pluginName is empty, return offending cookies from all enabled
  // plugins.
  // Note: this function screens out non-enabled plugins.
  Drupal.euccx.getOffendingCookies = function(pluginName) {
    var offendingCookies = [];
    var settings = Drupal.settings.euccx;
    var pluginNames = Drupal.euccx.getPluginNames();
    for (var i = 0; i < pluginNames.length; i++) {
      var currentPlugin = pluginNames[i];
      var cookies_handled = false;
      if (settings['plugins'] && settings['plugins'][currentPlugin] &&
        settings['plugins'][currentPlugin]['cookies_handled'])
        cookies_handled = settings['plugins'][currentPlugin]['cookies_handled'];

      // The below check translates to: if the plugin uses the cookies_handled
      // key AND it's an array AND (either this function was called for all
      // plugins or the plugin that it was called for is the one we're
      // checking right now), add the offending cookies to our array.
      if (cookies_handled && cookies_handled.constructor === Array &&
        (pluginName === '' || pluginName === pluginNames[i])) {
        offendingCookies = offendingCookies.concat(cookies_handled);
      }
    }
    return offendingCookies;
  }

  // We remove the offending cookies by setting their expiration date to the
  // past and their content to "null".
  // This is very similar to the functionality provided by the EU Cookie
  // Compliance module. The core differences are:
  // 1) Since the new version of jquery.cookie that you get through
  // jquery_update does not support removeCookie but instead allows you to
  // remove cookies by setting their value to null, we are using this new method
  // and are also setting the expiration date to the past.
  // 2) Since some cookies may be registered in both the subdomain but also the
  // root domain, we are blanket-replacing the cookie names that were defined by
  // our plugins. The user may see more cookies in the browser status, but all
  // these cookies will be "nulled out".
  // @todo: maybe find a way to make this so that it actually replaces the
  // cookies only when they exist (for now I don't have time to implement and
  // test it).
  Drupal.behaviors.euccxRemoveOffendingCookies = {
    attach: function (context, settings) {
      $(document).ready(function() {
        var status = Drupal.euccx.getCookieStatus();
        var offendingCookies = [];
        // Check if there is an EU Cookie Compliance cookie. If there is not,
        // all cookies defined in our plugins, should be considered offending
        // cookies because the user has not agreed to anything yet.
        if (!status['cookie-eucc']) {
          offendingCookies = Drupal.euccx.getOffendingCookies('');
        } 
        // If there is no euccx cookies but there is an eucc cookie, it means
        // that the user agreed to everything. However, if there is an eucc and
        // an euccx cookie, we need to dig a little deeper.
        else if (status['cookie-euccx']) {
          var pluginNames = Drupal.euccx.getPluginNames();
          for (var i = 0; i < pluginNames.length; i++) {
            var currentPlugin = pluginNames[i];
            // We only push the offending cookies of the current plugin, if the
            // plugin name is absent from our cookie.
            if (status['cookie-euccx'].indexOf(currentPlugin) === -1) {
              offendingCookies = 
                Drupal.euccx.getOffendingCookies(currentPlugin);
            }
          }
        }
        // Loop through any offending cookie names that we've found above 
        for (var i = 0; i < offendingCookies.length; i++) {
          var hostname = window.location.hostname;
          var index = 0;
          var ctd = offendingCookies[i];
          while (hostname !== '') {
            $.cookie(ctd, null, { 
              expires: -5,
              domain: '.' + hostname, 
              path: '/' }
            );
            $.cookie(ctd, null, { 
              expires: -5,
              domain: hostname, 
              path: '/' }
            );
            index = hostname.indexOf('.');
            hostname = (index === -1) ? '' : hostname.substring(index + 1);
          }
        }
      });
    }
  }
})(jQuery);
